﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using UnboundColumnUpdatingHierarchy.Models;
using Infragistics.Web.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace UnboundColumnUpdatingHierarchy.Controllers
{
    public class HomeController : Controller
    {
        public static AdventureWorksEntities context = new AdventureWorksEntities();
        Dictionary<object, object> peopleNames = context.People.Take(290).ToDictionary(x => (object)x.BusinessEntityID, x => (object)(x.FirstName + " " + x.LastName));
        public ActionResult Index()
        {
            ViewBag.Message = "This demo demonstrates the Ignite UI Grid's Unbound Column feature.";

            return View();
        }

        public ActionResult About()
        {
            ViewBag.Message = "This demo demonstrates the Ignite UI Grid's Unbound Column feature.";

            return View();
        }

        public ActionResult Bonus()
        {
            ViewBag.Message = "Below the grid demonstates Unbound Columns in both parent('Employees') and child ('Total/Max') layouts. Total's template includes both the calculated value and the applied max value." 
                + "Child layout unbound column values are applied using formula for Total and a combination of setting unbound values for Max and using a formula to supply defaults for those records where nore was added imitating the effect of server-side merging.";

            var deps = context.Departments.ToList().ConvertAll(x => x.ToDTO()).AsQueryable();
            return View(deps);
        }

        public ActionResult Updating()
        {
            ViewBag.Message = "Below the grid has a Names Unbound column and calculating the values for Total columns. Combined with the Updating feature and additional client side event handling to properly refresh formulas on updates. Besides formula, 'Total' has a template applied for conditional formatting. Below the grid are buttons to show the current array of transactions for the grid in order to demonstrate that using SetUnboundValues method will not generate transactions and therefore saving will have no effect. To properly set a value with Updating in mind the setCellValue method is used.";

            ViewBag.people = context.People.OrderBy(p => p.BusinessEntityID).Take(25).Select(p => p.FirstName + " " + p.LastName).ToList();
            return View();
        }

        public ActionResult HierarchicalSetValue()
        {
            ViewBag.Message = "Demonstrates Unbound columns in both parent('Employees') and child ('Counter') layouts in the Ignite UI jQuery grid. Buttons below offer use of the client side API to set Unbound values for the parent or all already expanded(created) child layouts.";
            var departments = context.Departments.ToList().ConvertAll(x => x.ToDTO()).AsQueryable();
            return View(departments);
        }

        public ActionResult MVCHierarchicalLoD()
        {
            ViewBag.Message = "Demonstrates Unbound columns in both parent('Employees') and child ('name and Counter') in the Ignite UI jQuery grid defined using the server-side grim model. The GetData() method is used to provide Load On demand functionality as well as handle Unbound Values.";
            var departments = context.Departments.ToList().ConvertAll(x => x.ToDTO()).AsQueryable();
            GridModel grid = GenerateGridModel();
            grid.DataSource = departments;
            return View(grid);
        }

        public JsonResult BindChild(string path, string layout)
        {
            GridModel viewModel = GenerateGridModel();
            AdventureWorksEntities context = new AdventureWorksEntities();
            viewModel.DataSource = context.Employees.ToList().ConvertAll(eh => eh.ToDTO()).AsQueryable();
            return viewModel.GetData(path, layout);
        }

        public ActionResult EditingTemplate()
        {
            ViewBag.Message = "Demonstrates Unbound columns in the Ignite UI jQuery grid defined using the MVC helper and with merging unbound values on to provide empty fields. Editing is enabled as the chidl layouts have template on the Unbound 'FUll Name' column that is entirely constructed of the two name fields and updates when you edit them out-of-the-box.";
            AdventureWorksEntities context = new AdventureWorksEntities();
            var employees = context.Employees.ToList().ConvertAll(e => e.ToHrcDTO());
            return View(employees.AsQueryable());
        }

        private GridModel GenerateGridModel()
        {
            GridModel grid = new GridModel();
            grid.Height = "500px";
            grid.AutoCommit = false;
            grid.PrimaryKey = "DepartmentID";
           
            grid.AutoGenerateColumns = false;
            grid.AutoGenerateLayouts = false;
            GridColumnLayoutModel childLayout = new GridColumnLayoutModel();

            //settings
            grid.LoadOnDemand = true;
            childLayout.Key = "Employees";
            childLayout.PrimaryKey = "BusinessEntityID";
            childLayout.ForeignKey = "DepartmentID";
            childLayout.MergeUnboundColumns = true;
            
            childLayout.AutoGenerateColumns = false;
            childLayout.AutoGenerateLayouts = false;
         
            childLayout.DataSourceUrl = Url.Action("BindChild");

            childLayout.Columns = new List<GridColumn>();
            childLayout.Columns.Add(new GridColumn()
            {
                Key = "BusinessEntityID",
                HeaderText = "B.E. Id",
                DataType = "number"
            });

            childLayout.Columns.Add(new GridColumn()
            {
                Key = "DepartmentID",
                HeaderText = "Dep id",
                DataType = "number",
                Hidden = true
            });

            childLayout.Columns.Add(new UnboundColumn()
            {
                Key = "name",
                HeaderText = "name",
                DataType = "string",
            });

            childLayout.Columns.Add(new GridColumn()
            {
                Key = "VacationHours",
                HeaderText = "Vacation Hours",
                DataType = "number"
            });
            childLayout.Columns.Add(new GridColumn()
            {
                Key = "SickLeaveHours",
                HeaderText = "Sick Leave Hours",
                DataType = "number"
            });
            childLayout.Columns.Add(new UnboundColumn()
            {
                Key = "counter",
                HeaderText = "Counter",
                DataType = "number"
            });
            childLayout.SetUnboundValues("counter", new List<object>() { 1, 2, 3, 4, 5, 6, 7, 8, 9 });
            childLayout.SetUnboundValues("name", peopleNames);

            grid.Columns = new List<GridColumn>();
            grid.Columns.Add(new GridColumn()
            {
                Key = "DepartmentID",
                HeaderText = "DEP Id",
                DataType = "number"
            });
            grid.Columns.Add(new GridColumn()
            {
                Key = "Name",
                HeaderText = "Name",
                DataType = "string"
            });

            grid.Columns.Add(new GridColumn()
            {
                Key = "GroupName",
                HeaderText = "Group Name",
                DataType = "string"
            });

            grid.Columns.Add(new GridColumn()
            {
                Key = "ModifiedDate",
                HeaderText = "Modified Date",
                DataType = "date"
            });
            grid.Columns.Add(new UnboundColumn()
            {
                Key = "employees",
                HeaderText = "Employees",
                DataType = "number",
                UnboundValues = new List<object>() { 7, 3, 18, 9, 12, 4, 180, 5, 6 }
            });

            grid.ColumnLayouts.Add(childLayout);
            return grid;
        }

        public JsonResult Employees()
        {
            AdventureWorksEntities context = new AdventureWorksEntities();
            var employees = context.Employees.Take(25).ToList().ConvertAll(e => e.ToDTO());
            return Json(employees, JsonRequestBehavior.AllowGet);
        }

        public JsonResult UpdateUrl()
        {
            var transactions = JsonConvert.DeserializeObject<List<Transaction<JObject>>>(Request.Form["ig_transactions"]);
            //  NOTE: JSON.NET is quite capable to cast the transacrion row to EmployeeDTO instead as it will ignore unmatching properties:
            //      var transactions = JsonConvert.DeserializeObject<List<Transaction<EmployeeDTO>>>(Request.Form["ig_transactions"]);
            //  form where you can assign the changed object directly but will have to pick out additional changes in some other way
            AdventureWorksEntities context = new AdventureWorksEntities();
            foreach (Transaction<JObject> transaction in transactions)
            {
                if (transaction.type == "cell" && transaction.col == "name") //cell edit(name in this demo only)!
                {
                    int id = int.Parse(transaction.rowId);
                    var person = context.People.Where(x => x.BusinessEntityID == id).Single();
                    person.FirstName = transaction.value.Split(' ').First();
                    person.LastName = transaction.value.Split(' ').Last();
                }
                else
                {
                    int id = (int)transaction.row["BusinessEntityID"];
                    // take id in advance as accessing it is not a canonical method and then find the data records 
                    var employee = context.Employees.Where(x => x.BusinessEntityID == id).Single();
                    var person = context.People.Where(x => x.BusinessEntityID == id).Single();
                    if (transaction.type == "row") //edit!
                    {
                        employee.VacationHours = (short)transaction.row["VacationHours"];
                        employee.SickLeaveHours = (short)transaction.row["SickLeaveHours"];
                        employee.LoginID = (string)transaction.row["LoginID"];
                        person.FirstName = transaction.row["name"].ToString().Split(' ').First();
                        person.LastName = transaction.row["name"].ToString().Split(' ').Last();
                    }
                    if (transaction.type == "delete") //delete!
                    {
                        context.Employees.DeleteObject(employee);
                    }
                }
            }
            context.SaveChanges();
            Response.Write("{\"Success\": true}");
            return new JsonResult();
        }
    }
}
